@@ -1,3 +1,4 @@
+use async_trait::async_trait;
 use codec::{Decode, Encode};
 use log::{debug, error};
 use parking_lot::Mutex;
@@ -11,7 +12,7 @@ use futures::{
 };
 
 use std::{
-    cell::Cell,
+    cell::{Cell, RefCell},
     collections::{hash_map::DefaultHasher, HashMap},
     hash::Hasher as StdHasher,
     pin::Pin,
@@ -21,18 +22,14 @@ use std::{
 };
 
 use crate::{
-    alerts,
-    config::{exponential_slowdown, Config, DelayConfig},
-    member::{NotificationIn, NotificationOut},
-    network,
+    exponential_slowdown, run_session,
+    runway::{NotificationIn, NotificationOut},
     units::{Unit, UnitCoord},
-    DataIO as DataIOT, Hasher, Index, KeyBox as KeyBoxT, MultiKeychain as MultiKeychainT,
-    Network as NetworkT, NodeCount, NodeIndex, OrderedBatch,
-    PartialMultisignature as PartialMultisignatureT, SpawnHandle,
+    Config, DataIO as DataIOT, DelayConfig, Hasher, Index, KeyBox as KeyBoxT,
+    MultiKeychain as MultiKeychainT, Network as NetworkT, NodeCount, NodeIndex, OrderedBatch,
+    PartialMultisignature as PartialMultisignatureT, Recipient, Round, SpawnHandle, TaskHandle,
 };
 
-use crate::member::Member;
-
 pub fn init_log() {
     let _ = env_logger::builder()
         .filter_level(log::LevelFilter::max())
@@ -40,9 +37,27 @@ pub fn init_log() {
         .try_init();
 }
 
+pub fn gen_config(node_ix: NodeIndex, n_members: NodeCount) -> Config {
+    let delay_config = DelayConfig {
+        tick_interval: Duration::from_millis(5),
+        requests_interval: Duration::from_millis(50),
+        unit_broadcast_delay: Arc::new(|t| exponential_slowdown(t, 100.0, 1, 3.0)),
+        //100, 100, 300, 900, 2700, ...
+        unit_creation_delay: Arc::new(|t| exponential_slowdown(t, 50.0, usize::MAX, 1.000)),
+        //50, 50, 50, 50, ...
+    };
+    Config {
+        node_ix,
+        session_id: 0,
+        n_members,
+        delay_config,
+        max_round: 5000,
+    }
+}
+
 // A hasher from the standard library that hashes to u64, should be enough to
 // avoid collisions in testing.
-#[derive(PartialEq, Eq, Clone, Debug)]
+#[derive(PartialEq, Eq, Hash, Clone, Debug)]
 pub struct Hasher64;
 
 impl Hasher for Hasher64 {
@@ -55,6 +70,8 @@ impl Hasher for Hasher64 {
     }
 }
 
+pub(crate) type Hash64 = <Hasher64 as Hasher>::Hash;
+
 // This struct allows to create a Hub to interconnect several instances of the Consensus engine, without
 // requiring the Member wrapper. The Hub notifies all connected instances about newly created units and
 // is able to answer unit requests as well. WrongControlHashes are not supported, which means that this
@@ -98,7 +115,7 @@ impl HonestHub {
             "Must connect to all nodes before running the hub."
         );
         for (_ix, tx) in self.ntfct_in_txs.iter() {
-            tx.unbounded_send(ntfct.clone()).unwrap();
+            tx.unbounded_send(ntfct.clone()).ok();
         }
     }
 
@@ -107,12 +124,12 @@ impl HonestHub {
             .ntfct_in_txs
             .get(&node_ix)
             .expect("Must connect to all nodes before running the hub.");
-        let _ = tx.unbounded_send(ntfct);
+        tx.unbounded_send(ntfct).expect("Channel should be open");
     }
 
     fn on_notification(&mut self, node_ix: NodeIndex, ntfct: NotificationOut<Hasher64>) {
         match ntfct {
-            NotificationOut::CreatedPreUnit(pu) => {
+            NotificationOut::CreatedPreUnit(pu, _parent_hashes) => {
                 let hash = pu.using_encoded(Hasher64::hash);
                 let u = Unit::new(pu, hash);
                 let coord = UnitCoord::new(u.round(), u.creator());
@@ -180,55 +197,73 @@ impl Future for HonestHub {
 }
 
 #[derive(Clone)]
-pub(crate) struct Spawner {
-    handles: Arc<Mutex<Vec<tokio::task::JoinHandle<()>>>>,
-}
+pub struct Spawner {}
 
 impl SpawnHandle for Spawner {
     fn spawn(&self, _name: &str, task: impl Future<Output = ()> + Send + 'static) {
-        self.handles.lock().push(tokio::spawn(task))
+        tokio::spawn(task);
+    }
+
+    fn spawn_essential(
+        &self,
+        _: &str,
+        task: impl Future<Output = ()> + Send + 'static,
+    ) -> TaskHandle {
+        let (res_tx, res_rx) = oneshot::channel();
+        tokio::spawn(async move {
+            task.await;
+            res_tx.send(()).expect("We own the rx.");
+        });
+        Box::pin(async move { res_rx.await.map_err(|_| ()) })
     }
 }
 
 impl Spawner {
-    pub(crate) async fn wait(&self) {
-        for h in self.handles.lock().iter_mut() {
-            let _ = h.await;
-        }
-    }
-    pub(crate) fn new() -> Self {
-        Spawner {
-            handles: Arc::new(Mutex::new(Vec::new())),
-        }
+    pub fn new() -> Self {
+        Spawner {}
     }
 }
 
-pub(crate) type NetworkData = crate::NetworkData<Hasher64, Data, Signature, PartialMultisignature>;
+impl Default for Spawner {
+    fn default() -> Self {
+        Spawner::new()
+    }
+}
+
+pub type NetworkData = crate::NetworkData<Hasher64, Data, Signature, PartialMultisignature>;
 type NetworkReceiver = UnboundedReceiver<(NetworkData, NodeIndex)>;
 type NetworkSender = UnboundedSender<(NetworkData, NodeIndex)>;
 
-pub(crate) struct Network {
+pub struct Network {
     rx: NetworkReceiver,
     tx: NetworkSender,
     peers: Vec<NodeIndex>,
     index: NodeIndex,
 }
 
+impl Network {
+    pub fn index(&self) -> NodeIndex {
+        self.index
+    }
+}
+
 #[async_trait::async_trait]
 impl NetworkT<Hasher64, Data, Signature, PartialMultisignature> for Network {
-    type Error = ();
-
-    fn broadcast(&self, data: NetworkData) -> Result<(), Self::Error> {
-        for peer in self.peers.iter() {
-            if *peer != self.index {
-                self.send(data.clone(), *peer)?;
+    fn send(&self, data: NetworkData, recipient: Recipient) {
+        use Recipient::*;
+        match recipient {
+            Node(node) => self
+                .tx
+                .unbounded_send((data, node))
+                .expect("send on channel should work"),
+            Everyone => {
+                for peer in self.peers.iter() {
+                    if *peer != self.index {
+                        self.send(data.clone(), Node(*peer));
+                    }
+                }
             }
         }
-        Ok(())
-    }
-
-    fn send(&self, data: NetworkData, node: NodeIndex) -> Result<(), Self::Error> {
-        self.tx.unbounded_send((data, node)).map_err(|_| ())
     }
 
     async fn next_event(&mut self) -> Option<NetworkData> {
@@ -241,35 +276,34 @@ struct Peer {
     rx: NetworkReceiver,
 }
 
-#[derive(Clone)]
-pub(crate) struct UnreliableRouter {
-    peers: Arc<Mutex<HashMap<NodeIndex, Peer>>>,
-    peer_list: Arc<Mutex<Vec<NodeIndex>>>,
-    hook_list: Arc<Mutex<Vec<Box<dyn NetworkHook + Send>>>>,
+pub struct UnreliableRouter {
+    peers: RefCell<HashMap<NodeIndex, Peer>>,
+    peer_list: Vec<NodeIndex>,
+    hook_list: RefCell<Vec<Box<dyn NetworkHook>>>,
     reliability: f64, //a number in the range [0, 1], 1.0 means perfect reliability, 0.0 means no message gets through
 }
 
 impl UnreliableRouter {
     pub(crate) fn new(peer_list: Vec<NodeIndex>, reliability: f64) -> Self {
         UnreliableRouter {
-            peers: Arc::new(Mutex::new(HashMap::new())),
-            peer_list: Arc::new(Mutex::new(peer_list)),
-            hook_list: Arc::new(Mutex::new(Vec::new())),
+            peers: RefCell::new(HashMap::new()),
+            peer_list,
+            hook_list: RefCell::new(Vec::new()),
             reliability,
         }
     }
 
-    pub(crate) fn add_hook<HK: NetworkHook + 'static>(&self, hook: HK) {
-        self.hook_list.lock().push(Box::new(hook));
+    pub fn add_hook<HK: NetworkHook + 'static>(&mut self, hook: HK) {
+        self.hook_list.borrow_mut().push(Box::new(hook));
     }
 
-    pub(crate) fn connect_peer(&self, peer: NodeIndex) -> Network {
+    pub(crate) fn connect_peer(&mut self, peer: NodeIndex) -> Network {
         assert!(
-            self.peer_list.lock().iter().any(|p| *p == peer),
+            self.peer_list.iter().any(|p| *p == peer),
             "Must connect a peer in the list."
         );
         assert!(
-            !self.peers.lock().contains_key(&peer),
+            !self.peers.borrow().contains_key(&peer),
             "Cannot connect a peer twice."
         );
         let (tx_in_hub, rx_in_hub) = unbounded();
@@ -278,11 +312,11 @@ impl UnreliableRouter {
             tx: tx_out_hub,
             rx: rx_in_hub,
         };
-        self.peers.lock().insert(peer, peer_entry);
+        self.peers.borrow_mut().insert(peer, peer_entry);
         Network {
             rx: rx_out_hub,
             tx: tx_in_hub,
-            peers: self.peer_list.lock().clone(),
+            peers: self.peer_list.clone(),
             index: peer,
         }
     }
@@ -290,11 +324,13 @@ impl UnreliableRouter {
 
 impl Future for UnreliableRouter {
     type Output = ();
-    fn poll(self: Pin<&mut Self>, cx: &mut Context<'_>) -> Poll<Self::Output> {
+    fn poll(mut self: Pin<&mut Self>, cx: &mut Context<'_>) -> Poll<Self::Output> {
+        let this = &mut self;
         let mut disconnected_peers: Vec<NodeIndex> = Vec::new();
         let mut buffer = Vec::new();
-        for (peer_id, peer) in self.peers.lock().iter_mut() {
+        for (peer_id, peer) in this.peers.borrow_mut().iter_mut() {
             loop {
+                // this call is responsible for waking this Future
                 match peer.rx.poll_next_unpin(cx) {
                     Poll::Ready(Some(msg)) => {
                         buffer.push((*peer_id, msg));
@@ -310,17 +346,17 @@ impl Future for UnreliableRouter {
             }
         }
         for peer_id in disconnected_peers {
-            self.peers.lock().remove(&peer_id);
+            this.peers.borrow_mut().remove(&peer_id);
         }
         for (sender, (mut data, recipient)) in buffer {
             let rand_sample = rand::random::<f64>();
-            if rand_sample > self.reliability {
+            if rand_sample > this.reliability {
                 debug!("Simulated network fail.");
                 continue;
             }
-            let mut peers = self.peers.lock();
-            if let Some(peer) = peers.get_mut(&recipient) {
-                for hook in self.hook_list.lock().iter_mut() {
+
+            if let Some(peer) = this.peers.borrow().get(&recipient) {
+                for hook in this.hook_list.borrow_mut().iter_mut() {
                     hook.update_state(&mut data, sender, recipient);
                 }
                 peer.tx
@@ -328,13 +364,16 @@ impl Future for UnreliableRouter {
                     .expect("channel should be open");
             }
         }
-
-        Poll::Pending
+        if this.peers.borrow().is_empty() {
+            Poll::Ready(())
+        } else {
+            Poll::Pending
+        }
     }
 }
 
-pub(crate) trait NetworkHook: Send {
-    fn update_state(&self, data: &mut NetworkData, sender: NodeIndex, recipient: NodeIndex);
+pub trait NetworkHook: Send {
+    fn update_state(&mut self, data: &mut NetworkData, sender: NodeIndex, recipient: NodeIndex);
 }
 
 #[derive(Clone)]
@@ -362,9 +401,8 @@ impl AlertHook {
 }
 
 impl NetworkHook for AlertHook {
-    fn update_state(&self, data: &mut NetworkData, sender: NodeIndex, recipient: NodeIndex) {
-        use alerts::AlertMessage::*;
-        use network::NetworkDataInner::*;
+    fn update_state(&mut self, data: &mut NetworkData, sender: NodeIndex, recipient: NodeIndex) {
+        use crate::{alerts::AlertMessage::*, network::NetworkDataInner::*};
         if let crate::NetworkData(Alert(ForkAlert(_))) = data {
             *self
                 .alerts_sent_by_connection
@@ -376,22 +414,23 @@ impl NetworkHook for AlertHook {
 }
 
 #[derive(Debug, Clone, PartialEq, Eq, Encode, Decode, Hash)]
-pub(crate) struct Data {
+pub struct Data {
     coord: UnitCoord,
     variant: u32,
 }
 
 impl Data {
+    #[cfg(test)]
     pub(crate) fn new(coord: UnitCoord, variant: u32) -> Self {
         Data { coord, variant }
     }
 }
 
-#[derive(Debug, Clone, PartialEq, Eq, Encode, Decode)]
-pub(crate) struct Signature {}
+#[derive(Debug, Clone, PartialEq, Eq, Hash, Encode, Decode)]
+pub struct Signature {}
 
-#[derive(Debug, Clone, PartialEq, Eq, Encode, Decode)]
-pub(crate) struct PartialMultisignature {
+#[derive(Debug, Clone, PartialEq, Eq, Hash, Encode, Decode)]
+pub struct PartialMultisignature {
     signed_by: Vec<NodeIndex>,
 }
 
@@ -411,7 +450,7 @@ impl PartialMultisignatureT for PartialMultisignature {
 
 pub(crate) struct DataIO {
     ix: NodeIndex,
-    round_counter: Cell<usize>,
+    round_counter: Cell<Round>,
     tx: UnboundedSender<OrderedBatch<Data>>,
 }
 
@@ -422,12 +461,6 @@ impl DataIOT<Data> for DataIO {
         self.round_counter.set(self.round_counter.get() + 1);
         Data { coord, variant: 0 }
     }
-    fn check_availability(
-        &self,
-        _: &Data,
-    ) -> Option<Pin<Box<dyn Future<Output = Result<(), Self::Error>> + Send>>> {
-        None
-    }
     fn send_ordered_batch(&mut self, data: OrderedBatch<Data>) -> Result<(), ()> {
         self.tx.unbounded_send(data).map_err(|e| {
             error!(target: "data-io", "Error when sending data from DataIO {:?}.", e);
@@ -465,11 +498,18 @@ impl Index for KeyBox {
     }
 }
 
+#[async_trait]
 impl KeyBoxT for KeyBox {
     type Signature = Signature;
-    fn sign(&self, _msg: &[u8]) -> Signature {
+
+    fn node_count(&self) -> NodeCount {
+        self.count
+    }
+
+    async fn sign(&self, _msg: &[u8]) -> Signature {
         Signature {}
     }
+
     fn verify(&self, _msg: &[u8], _sgn: &Signature, _index: NodeIndex) -> bool {
         true
     }
@@ -486,59 +526,65 @@ impl MultiKeychainT for KeyBox {
     }
 }
 
-pub(crate) fn gen_config(node_ix: NodeIndex, n_members: NodeCount) -> Config {
-    let delay_config = DelayConfig {
-        tick_interval: Duration::from_millis(5),
-        requests_interval: Duration::from_millis(50),
-        unit_broadcast_delay: Arc::new(|t| exponential_slowdown(t, 100.0, 1, 3.0)),
-        //100, 100, 300, 900, 2700, ...
-        unit_creation_delay: Arc::new(|t| exponential_slowdown(t, 50.0, usize::MAX, 1.000)),
-        //50, 50, 50, 50, ...
-    };
-    Config {
-        node_ix,
-        session_id: 0,
-        n_members,
-        delay_config,
-        rounds_margin: 200,
-        max_units_per_alert: 200,
-        max_round: 5000,
-    }
+pub(crate) async fn run_honest_member<
+    N: 'static + NetworkT<Hasher64, Data, Signature, PartialMultisignature>,
+>(
+    config: Config,
+    network: N,
+    data_io: DataIO,
+    keybox: KeyBox,
+    spawn_handle: Spawner,
+    exit: oneshot::Receiver<()>,
+) {
+    run_session(config, network, data_io, keybox, spawn_handle, exit).await
 }
 
-pub(crate) type HonestMember<'a> = Member<'a, Hasher64, Data, DataIO, KeyBox, Spawner>;
-
-pub(crate) fn configure_network(
-    n_members: usize,
+pub fn configure_network(
+    n_members: NodeCount,
     reliability: f64,
-) -> (UnreliableRouter, Vec<Option<Network>>) {
-    let peer_list = (0..n_members).map(NodeIndex).collect();
-
-    let router = UnreliableRouter::new(peer_list, reliability);
+) -> (UnreliableRouter, Vec<Network>) {
+    let peer_list = n_members.into_iterator().collect();
+    let mut router = UnreliableRouter::new(peer_list, reliability);
     let mut networks = Vec::new();
-    for ix in 0..n_members {
-        let network = router.connect_peer(NodeIndex(ix));
-        networks.push(Some(network));
+    for ix in n_members.into_iterator() {
+        let network = router.connect_peer(ix);
+        networks.push(network);
     }
     (router, networks)
 }
 
-pub(crate) fn spawn_honest_member(
+pub fn spawn_honest_member(
     spawner: Spawner,
-    ix: usize,
-    n_members: usize,
-    network: Network,
-) -> (UnboundedReceiver<OrderedBatch<Data>>, oneshot::Sender<()>) {
-    let node_index = NodeIndex(ix);
+    node_index: NodeIndex,
+    n_members: NodeCount,
+    network: impl 'static + NetworkT<Hasher64, Data, Signature, PartialMultisignature>,
+) -> (
+    UnboundedReceiver<OrderedBatch<Data>>,
+    oneshot::Sender<()>,
+    TaskHandle,
+) {
     let (data_io, rx_batch) = DataIO::new(node_index);
-    let config = gen_config(node_index, n_members.into());
+    let config = gen_config(node_index, n_members);
     let (exit_tx, exit_rx) = oneshot::channel();
     let spawner_inner = spawner.clone();
     let member_task = async move {
-        let keybox = KeyBox::new(NodeCount(n_members), node_index);
-        let member = HonestMember::new(data_io, &keybox, config, spawner_inner.clone());
-        member.run_session(network, exit_rx).await;
+        let keybox = KeyBox::new(n_members, node_index);
+        run_honest_member(
+            config,
+            network,
+            data_io,
+            keybox,
+            spawner_inner.clone(),
+            exit_rx,
+        )
+        .await
     };
-    spawner.spawn("member", member_task);
-    (rx_batch, exit_tx)
+    let handle = spawner.spawn_essential("member", member_task);
+    (rx_batch, exit_tx, handle)
+}
+
+pub fn complete_oneshot<T: std::fmt::Debug>(t: T) -> oneshot::Receiver<T> {
+    let (tx, rx) = oneshot::channel();
+    tx.send(t).unwrap();
+    rx
 }
